import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import controlP5.*; 
import proxml.*; 
import processing.serial.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class RoboBrrd_Dashboard extends PApplet {

/* RoboBrrd Dashboard v2
 * ------------------------
 *
 * Control your RoboBrrd and set its servo positions!
 *
 * To make RoboBrrd work with this app, be sure to upload the Boilerplate
 * code it it. It's included as an example in the RoboBrrd Arduino library!
 * For more information, please see here:
 * --> http://robobrrd.com/rb3d/programming1.php
 *
 * For more information about how to use the Dashboard app, check out its
 * webpage here:
 * --> http://robobrrd.com/dashboard
 *
 * Released under the MIT license, see license.txt for more info.
 *
 * By Erin Kennedy for RoboBrrd
 * Friday October 31, 2014
 */





static boolean DEBUG = false;

// -- XML -- //
XMLElement settingsXml;
XMLInOut xmlInOut;

// -- GUI -- //
ControlP5 cp5;
ControlP5 cp5_v2;
DropdownList d1;

PFont titleFont;
ControlFont titleFontc5;


// -- POSITIONS -- //
int beakOpenPos = 180;
int beakClosedPos = 0;

int lwingUpPos = 180;
int lwingDownPos = 0;
int lwingHomePos = 90;

int rwingUpPos = 0;
int rwingDownPos = 180;
int rwingHomePos = 90;

// -- SLIDERS -- //
int beakPos;
int leftPos;
int rightPos;

// -- LEDs -- //
// these are true but set to false at the end of setup(), cP5 is weird
int redVal = 0;
int greenVal = 0;
int blueVal = 0;

int redVal_r = 0;
int greenVal_r = 0;
int blueVal_r = 0;

boolean eye_press = false;

boolean eye_changed = false;
long last_eye_change = 0;

int redOn = color(255, 0, 0);  
int greenOn = color(0, 255, 0);
int blueOn = color(0, 0, 255);

int redOff = color(100, 20, 20);  
int greenOff = color(20, 100, 20);
int blueOff = color(20, 20, 100);

int white = color(255, 255, 255);
int black = color(20, 20, 20);

// note: CColor(foreground (hover), background (normal), active (press down), caption label, value label )
CColor red1Colour = new CColor(redOn, redOff, color(255, 100, 100), color(255, 255, 255), color(255, 255, 255));
CColor green1Colour = new CColor(greenOn, greenOff, color(100, 255, 100), color(255, 255, 255), color(255, 255, 255));
CColor blue1Colour = new CColor(blueOn, blueOff, color(100, 100, 255), color(255, 255, 255), color(255, 255, 255));
CColor mix1Colour = new CColor(white, black, color(200, 200, 200), color(140, 140, 140), color(0, 0, 0));


// -- LDRs -- //
int leftLDRReading = 0;
int rightLDRReading = 0;
boolean refresh_ldrs = false;
long last_ldr = 0;


// -- THEMES -- //
boolean estop = false;
long last_estop_send = 0;

int chosenTheme = 0;
int readTheme = 0;
int numThemes = 3;

int[] buttonFg = { color(0, 139, 214), color(30, 255, 0), 0xffFF7300 };
int[] buttonBg = { color(36, 99, 133), color(18, 18, 18), 0xff0077FF };
int[] buttonAct = { color(255, 217, 0), color(112, 189, 102), 0xff9000FF };
int[] captionCol = { color(255, 255, 255), color(255, 255, 255), 0xff00FF08 };
int[] valueCol = { color(255, 255, 255), color(255, 255, 255), 0xff00FF08 };

int[] titleCol = { color(255, 234, 0), color(30, 255, 0), 0xffAA00FF };
int[] bgCol = { color(25, 186, 255), color(0, 0, 0), 0xff008CFF };

boolean[] bgImg = { true, true, true };
String[] imgPaths = { "theme1bg.jpg", "theme2bg.jpg", "theme3bg.jpg" };
PImage[] bgImgs = new PImage[numThemes];

CColor buttonColour = new CColor(buttonFg[chosenTheme], buttonBg[chosenTheme], buttonAct[chosenTheme], captionCol[chosenTheme], valueCol[chosenTheme]);

CColor theme1Colour = new CColor(buttonFg[0], buttonBg[0], buttonAct[0], captionCol[0], valueCol[0]);
CColor theme2Colour = new CColor(buttonFg[1], buttonBg[1], buttonAct[1], captionCol[1], valueCol[1]);
CColor theme3Colour = new CColor(buttonFg[2], buttonBg[2], buttonAct[2], captionCol[2], valueCol[2]);

String[] allThemedElements = {"connect", "beakSlider", "leftSlider", "rightSlider", "beakopen", "beakclosed", "leftup", "leftdown", "lefthome", "rightup", "rightdown", "righthome", "beakopen_button", "beakclosed_button", "lwingup_button", "lwingdown_button", "lwinghome_button", "rwingup_button", "rwingdown_button", "rwinghome_button", "writevals_button", "enable_ldrs_button", "estop_button" };


// -- LET'S GOOOOO!!! :D -- //
public void setup() {

  size(1024, 680);
  noStroke();
  cp5 = new ControlP5(this);
  cp5_v2 = new ControlP5(this);
  cp5.setAutoDraw(false);
  cp5_v2.setAutoDraw(false);
  cp5.setMoveable(false);
  cp5_v2.setMoveable(false);
  
  titleFont = loadFont("Arista2.0Light-48.vlw");
  titleFontc5 = new ControlFont(titleFont, 30);
  
  println("-------------------- " + redVal + " " + redVal_r);
  
  startXml();
  
  initGui();

  // controlp5 is weird sometimes, this is just to fix the coloured sliders  
  redVal = redVal_r;
  greenVal = greenVal_r;
  blueVal = blueVal_r;
  estop = false;
  refresh_ldrs = false;
  
  ((Slider)cp5.getController("redSlider")).setValue(redVal);
  ((Slider)cp5.getController("greenSlider")).setValue(greenVal);
  ((Slider)cp5.getController("blueSlider")).setValue(blueVal);
 
  ((Slider)cp5.getController("leftSlider")).setRange(lwingDownPos, lwingUpPos);
  ((Slider)cp5.getController("rightSlider")).setRange(rwingDownPos, rwingUpPos);
  ((Slider)cp5.getController("beakSlider")).setRange(beakOpenPos, beakClosedPos);
  
  cp5.getController("leftSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("leftSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.getController("rightSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("rightSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.getController("beakSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("beakSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  refreshSliders();
  
  
  
  
  // -- OTHER -- //
  
  frame.setBackground(new java.awt.Color(0,0,0));

  for (int i=0; i<Serial.list().length; i++) {
    println(Serial.list()[i]);
    d1.addItem("" + Serial.list()[i], i);
  }
  
   
  chosenTheme = readTheme;
  updateTheme();

  
  for(int i=0; i<numThemes; i++) {
    bgImgs[i] = loadImage(dataPath(imgPaths[i]));
  }
  
  
}


// -- DRAW -- //
public void draw() {
  
  if(connected) {
    readData();  
  }
  
  drawBackground();
  
  cp5.draw();
  cp5_v2.draw();
  
  
  if(millis()-last_ldr >= 100 && refresh_ldrs == true) {
    transmit_action('@', 'I', 0, 0, '!');
    delay(20);
    transmit_action('@', 'J', 0, 0, '!');
    delay(20);
    last_ldr = millis();
  }
  drawSensors();
  
  
  if(millis()-last_eye_change >= 1000 && eye_changed == true) {
    saveXml();
    eye_changed = false;
  }
  
  if(millis()-last_estop_send >= 80 && estop == true) {
    transmit_action('#', 'O', 0, 0, '!');
    last_estop_send = millis();
  }
  
}


public void drawBackground() {
 
 background(bgCol[chosenTheme]);
  
  if(bgImg[chosenTheme]) {
    image(bgImgs[chosenTheme], 0, 0);
  }
  
}

public void drawSensors() {
 
 int lbar = (int)map(leftLDRReading, 0, 1023, 0, 250);
 int rbar = (int)map(rightLDRReading, 0, 1023, 0, 250);
  
  //println("rbar: " + rbar);
  
  // left ldr
  fill(buttonBg[chosenTheme]);
  rect(720, 310, 80, 250);
  
  fill(buttonFg[chosenTheme]);
  rect(720, 560, 80, -1*lbar); // 310 + 250 = 560
  
  
  // right ldr 
  fill(buttonBg[chosenTheme]);
  rect(820, 310, 80, 250);
  
  fill(buttonFg[chosenTheme]);
  rect(820, 560, 80, -1*rbar);
  
}


// -- ARDUINO -- //
Serial arduino;
int port = 99;
boolean connected = false;

// -- PARSING -- //
int msgLen = 20;
int msgIndex = 0;
char[] msg = new char[msgLen];
boolean reading = false;
boolean completed = false;
int receivedValue = 0;


public void readData() {
 
  if(connected) {
    // let's read the data
    char c;
    
    while(arduino.available() > 0) {
      
      c = arduino.readChar();
      
      if(c == '~' || c == '@' || c == '#' || c == '^' || c == '&') {
        reading = true;
        completed = false; 
      }
      
      if(reading) {
        if(DEBUG) println("readChar(): " + c + " msgIndex: " + msgIndex);
        
        msg[msgIndex++] = c;
      
        if(c == '!' || c == '?' || c == ';') {
          if(msgIndex >= 6-1) {
            completed = true;
          } else {
            if(DEBUG) println("promulgate error: received delimeter before the message was long enough"); 
          }
        }
        
        if(completed == true) {
          msgLen = msgIndex;
          msgIndex = 0;
          reading = false;
          parse_message();
          return;
        }
        
        if(msgIndex >= 20-1) {
          if(DEBUG) println("promulgate warning: index exceeded max message length");
          msgIndex = 0;
          msgLen = 0;
        }
    
      }
      
    }
    
    
  }
  
}


public void parse_message() {
  
  if(DEBUG) {
    println("parsing message now");
    for(int i=0; i<msgLen; i++) {
      print(msg[i]);
    }
    println("\n(end)");
  }
  
  char action = '0';
  char cmd = '0';
  int key_msg = 0;
  int val = 0;
  char delim = '0';
  
  String temp_key = "";
  String temp_val = "";
  int comma_index = 4;
  int delim_index = msgLen-1;
  
  // setting the action and command
  action = msg[0];
  cmd = msg[1];
  
  
  // finding the key
  for(int i=2; i<msgLen; i++) {
    
    if(msg[i] == ',') {
      comma_index = i;
      break;
    } else {
      temp_key = temp_key + msg[i];
    }
    
  }
  
  try {
    key_msg = (int)Integer.parseInt(temp_key);
  } catch(Exception e) {
    println("promulgate error: failed to parse int for key");
    println(e);
    return;
  }
  
  
  // finding the val
  for(int i=comma_index+1; i<msgLen-1; i++) {
    
    char c = msg[i];
    
    if(c == '!' || c == '?' || c == ';') {
      delim_index = i;
      break;
    } else {
      temp_val = temp_val + msg[i];
    }
    
  }
  
  try {
    val = (int)Integer.parseInt(temp_val);
  } catch(Exception e) {
    println("promulgate error: failed to parse int for val");
    println(e);
    return;
  }
  
  // setting the delimeter
  delim = msg[delim_index];
  
  
  // finally done!
  received_action(action, cmd, key_msg, val, delim);
  
}



// this is where we parse the received info
// from the Arduino!
public void received_action(char action, char cmd, int key_msg, int val, char delim) {
  
  if(DEBUG) {
    println("---RECEIVED ACTION!---");
    println("action: " + action);
    println("cmd: " + cmd);
    println("key: " + key_msg);
    println("val: " + val);
    println("delim: " + delim);
  }
  
  if(action == '#') {
    
    if(cmd == 'I') {
      leftLDRReading = val;
    } else if(cmd == 'J') {
      rightLDRReading = val;
    }
    
  }
  
}


public void transmit_complete() {
  if(DEBUG) println("message sent");
}


public void transmit_action(char action, char cmd, int key_msg, int val, char delim) {
  
  // filter out all of the actions that would move the servos, if estop is enabled
  if(estop) {
    
    if(action == '@') {
      if(cmd == 'S' || cmd == 'B' || cmd == 'R' || cmd == 'L') return;
      
    } else if(action == '#') {
      if(cmd != 'O' && key_msg != 0) return;
    }
    
  }
  
  if(key_msg > 256) {
    println("promulgate error: key has to be <= 256");
    return;
  }
  
  if(val > 1023) {
    println("promulgate error: val has to be <= 1023");
    return;
  }
  
  String s = (action + "" + cmd + "" + key_msg + "," + val + "" + delim);
  //println(s);
  if(connected) {
    arduino.write(s);
    transmit_complete();
  }
  
}



// -- XML HANDLING -- //

public void startXml() {
 
 xmlInOut = new XMLInOut(this);
  try{
    xmlInOut.loadElement(dataPath("settings.xml")); 
  }catch(Exception e){
    // derp?
  }
  
  
}


public void xmlEvent(XMLElement element){
  settingsXml = element;
  initXml();
}

public void initXml() {
  //settingsXml.printElementTree(" ");
  
  XMLElement servosXml;
  XMLElement beakXml;
  XMLElement lwingXml;
  XMLElement rwingXml;
  XMLElement themeXml;
  XMLElement position;
  XMLElement ledsXml;
  
  themeXml = settingsXml.getChild(0);
  readTheme = themeXml.getIntAttribute("num");

  servosXml = settingsXml.getChild(1);
  beakXml = servosXml.getChild(0);
  lwingXml = servosXml.getChild(1);
  rwingXml = servosXml.getChild(2);
  
    position = beakXml.getChild(0);
    beakOpenPos = position.getIntAttribute("up");
    beakClosedPos = position.getIntAttribute("down");
    
    position = lwingXml.getChild(0);
    lwingUpPos = position.getIntAttribute("up");
    lwingDownPos = position.getIntAttribute("down");
    lwingHomePos = position.getIntAttribute("home");
  
    position = rwingXml.getChild(0);
    rwingUpPos = position.getIntAttribute("up");
    rwingDownPos = position.getIntAttribute("down");
    rwingHomePos = position.getIntAttribute("home");

  ledsXml = settingsXml.getChild(2);
  redVal_r = ledsXml.getIntAttribute("red");
  greenVal_r = ledsXml.getIntAttribute("green");
  blueVal_r = ledsXml.getIntAttribute("blue");
  
  println("read xml" + redVal_r);
  
}

public void saveXml() {
  
  if(millis() <= 1000) return; // WTF
  
  XMLElement settingsXml0 = new XMLElement("settings");
 
  XMLElement themeXml0 = new XMLElement("theme");
  XMLElement servosXml0 = new XMLElement("servos");
  
  XMLElement ledsXml0 = new XMLElement("leds");
  
  themeXml0.addAttribute("num", chosenTheme);
  
  settingsXml0.addChild(themeXml0);
  
  //--
  XMLElement beakXml0 = new XMLElement("beak");
  XMLElement lwingXml0 = new XMLElement("lwing");
  XMLElement rwingXml0 = new XMLElement("rwing");
  
  XMLElement posXml0 = new XMLElement("position");
  
  posXml0.addAttribute("up", beakOpenPos);
  posXml0.addAttribute("down", beakClosedPos);
  
  beakXml0.addChild(posXml0);
  servosXml0.addChild(beakXml0);
  
  //--
  XMLElement posXml1 = new XMLElement("position");
  
  posXml1.addAttribute("up", lwingUpPos);
  posXml1.addAttribute("down", lwingDownPos);
  posXml1.addAttribute("home", lwingHomePos);
  
  lwingXml0.addChild(posXml1);
  servosXml0.addChild(lwingXml0);
  
  //--
  XMLElement posXml2 = new XMLElement("position");
  
  posXml2.addAttribute("up", rwingUpPos);
  posXml2.addAttribute("down", rwingDownPos);
  posXml2.addAttribute("home", rwingHomePos);
  
  rwingXml0.addChild(posXml2);
  servosXml0.addChild(rwingXml0);
  
  //--
  settingsXml0.addChild(servosXml0);
  
  ledsXml0.addAttribute("red", redVal);
  ledsXml0.addAttribute("green", greenVal);
  ledsXml0.addAttribute("blue", blueVal);
  
  settingsXml0.addChild(ledsXml0);
  
  xmlInOut.saveElement(settingsXml0, "settings.xml");
  
  
  println("wrote xml");
  
}

public void refreshSliders() {
 
 ((Slider)cp5.getController("redSlider")).setValue(redVal);
  ((Slider)cp5.getController("greenSlider")).setValue(greenVal);
  ((Slider)cp5.getController("blueSlider")).setValue(blueVal);
 
  ((Slider)cp5.getController("leftSlider")).setRange(lwingDownPos, lwingUpPos);
  ((Slider)cp5.getController("rightSlider")).setRange(rwingDownPos, rwingUpPos);
  ((Slider)cp5.getController("beakSlider")).setRange(beakOpenPos, beakClosedPos);

cp5.getController("beakSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("beakSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);

  cp5.getController("leftSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("leftSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.getController("rightSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("rightSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);

  
}



public void initGui() {
 
 
  // -- TITLE -- //
  
  Group g0 = cp5.addGroup("g0")
                .setPosition(45,10)
                .setWidth(955)
                .setHeight(60)
                .setBackgroundHeight(60)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
  cp5.addTextlabel("title")
     .setText("RoboBrrd Dashboard")
     .setPosition(5,10+8)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g0)
     .setFont(titleFontc5)
     ;
  
   cp5.addTextlabel("version")
     .setText("v2.0")
     .setPosition(925,45)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g0)
     ;
  
   d1 = cp5_v2.addDropdownList("serialports")
          .setPosition(360, 55)
          .setSize(200,150)
          .setColor(buttonColour)
          .setBarHeight(30)
          .setCaptionLabel("Serial Ports")
          ;
          
  customize(d1);
  
  cp5.addButton("connect")
     .setValue(0)
     .setPosition(530,15)
     .setSize(100,30)
     .setColor(buttonColour)
     .setGroup(g0)
     .setCaptionLabel("Connect!")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER);
     ;
  
  cp5.addButton("theme1_button")
     .setValue(0)
     .setPosition(850,20)
     .setSize(20,20)
     .setColor(theme1Colour)
     .setGroup(g0)
     .setCaptionLabel("")
     ;
  
  cp5.addButton("theme2_button")
     .setValue(0)
     .setPosition(875,20)
     .setSize(20,20)
     .setColor(theme2Colour)
     .setGroup(g0)
     .setCaptionLabel("")
     ;
  
  cp5.addButton("theme3_button")
     .setValue(0)
     .setPosition(900,20)
     .setSize(20,20)
     .setColor(theme3Colour)
     .setGroup(g0)
     .setCaptionLabel("")
     ;
     
  cp5.addButton("estop_button")
     .setValue(0)
     .setPosition(700,15)
     .setSize(100,30)
     .setGroup(g0)
     .setColor(buttonColour)
     .setCaptionLabel("Emergency Stop")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  
  // -- SERVOS -- //
  
  Group g1 = cp5.addGroup("g1")
                .setPosition(45,80)
                .setWidth(300)
                .setHeight(260)
                .setBackgroundHeight(260)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
  cp5.addTextlabel("servos")
     .setText("Servos")
     .setPosition(110,7)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g1)
     .setFont(titleFontc5)
     ;
  
  
  cp5.addSlider("beakSlider")
     .setPosition(25, 40)
     .setSize(50,200)
     .setColor(buttonColour)
     .setRange(beakClosedPos,beakOpenPos)
     .setValue(beakClosedPos)
     .setCaptionLabel("Beak")
     .setGroup(g1)
     ;
  
  cp5.getController("beakSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("beakSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);

  cp5.addSlider("leftSlider")
     .setPosition(120, 40)
     .setSize(50,200)
     .setColor(buttonColour)
     .setRange(lwingDownPos,lwingUpPos)
     .setValue(lwingHomePos)
     .setCaptionLabel("Left Wing")
     .setGroup(g1)
     ;
  
  cp5.getController("leftSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("leftSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.addSlider("rightSlider")
     .setPosition(220, 40)
     .setSize(50,200)
     .setColor(buttonColour)
     .setRange(rwingDownPos,rwingUpPos)
     .setValue(rwingHomePos)
     .setCaptionLabel("Right Wing")
     .setGroup(g1)
     ;
  
  cp5.getController("rightSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("rightSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);

  
  // -- POSITIONS -- //
  
 Group g2 = cp5.addGroup("g2")
                .setPosition(45,360)
                .setWidth(300)
                .setHeight(300)
                .setBackgroundHeight(300)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;


  PFont textfieldFont = createFont("arial",20);

  cp5.addTextlabel("positions")
     .setText("Positions")
     .setPosition(100,7)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g2)
     .setFont(titleFontc5)
     ;

  cp5.addTextfield("beakopen")
     .setPosition(20,40)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + beakOpenPos)
     .setCaptionLabel("Beak Open")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
  
  cp5.addTextfield("beakclosed")
     .setPosition(20,110)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + beakClosedPos)
     .setCaptionLabel("Beak Closed")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
  
 
   cp5.addTextfield("leftup")
     .setPosition(120,40)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + lwingUpPos)
     .setCaptionLabel("Left Up")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
  
  cp5.addTextfield("leftdown")
     .setPosition(120,110)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + lwingDownPos)
     .setCaptionLabel("Left Down")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
     
  cp5.addTextfield("lefthome")
     .setPosition(120,180)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + lwingHomePos)
     .setCaptionLabel("Left Home")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
  
  
  
  cp5.addTextfield("rightup")
     .setPosition(220,40)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + rwingUpPos)
     .setCaptionLabel("Right Up")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
  
  cp5.addTextfield("rightdown")
     .setPosition(220,110)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + rwingDownPos)
     .setCaptionLabel("Right Down")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
     
  cp5.addTextfield("righthome")
     .setPosition(220,180)
     .setSize(50,40)
     .setFont(textfieldFont)
     .setColor(buttonColour)
     .setGroup(g2)
     .setAutoClear(false)
     .setText("" + rwingHomePos)
     .setCaptionLabel("Right Home")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE)
     ;
     
  cp5.addButton("writevals_button")
     .setValue(0)
     .setPosition(75,250)
     .setSize(150,30)
     .setColor(buttonColour)
     .setGroup(g2)
     .setCaptionLabel("Write Values")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  
  // -- ACTIONS -- //
  
  Group g3 = cp5.addGroup("g3")
                .setPosition(360,80)
                .setWidth(240)
                .setHeight(580)
                .setBackgroundHeight(580)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
  
  cp5.addTextlabel("actions")
     .setText("Actions")
     .setPosition(70,7)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g3)
     .setFont(titleFontc5)
     ;
  
  cp5.addButton("beakopen_button")
     .setValue(0)
     .setPosition(40,40)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Beak Open")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  cp5.addButton("beakclosed_button")
     .setValue(0)
     .setPosition(40,100)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Beak Closed")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  cp5.addButton("lwingup_button")
     .setValue(0)
     .setPosition(40,180)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Left Wing Up")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  cp5.addButton("lwingdown_button")
     .setValue(0)
     .setPosition(40,240)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Left Wing Down")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  cp5.addButton("lwinghome_button")
     .setValue(0)
     .setPosition(40,300)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Left Wing Home")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  cp5.addButton("rwingup_button")
     .setValue(0)
     .setPosition(40,380)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Right Wing Up")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  cp5.addButton("rwingdown_button")
     .setValue(0)
     .setPosition(40,440)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Right Wing Down")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  cp5.addButton("rwinghome_button")
     .setValue(0)
     .setPosition(40,500)
     .setSize(150,50)
     .setColor(buttonColour)
     .setGroup(g3)
     .setCaptionLabel("Right Wing Home")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  // -- LEDs -- //
  
  Group g4 = cp5.addGroup("g4")
                .setPosition(620,80)
                .setWidth(380)
                .setHeight(180)
                .setBackgroundHeight(180)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
  cp5.addTextlabel("leds")
     .setText("Eyes")
     .setPosition(155,7)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g4)
     .setFont(titleFontc5)
     ;
  
  cp5.addSlider("redSlider")
     .setPosition(30, 40)
     .setSize(50,100)
     .setColor(red1Colour)
     .setRange(0,255)
     .setValue(redVal)
     .setCaptionLabel("Red")
     .setGroup(g4)
     ;
  
  cp5.getController("redSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("redSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.addSlider("greenSlider")
     .setPosition(100, 40)
     .setSize(50,100)
     .setColor(green1Colour)
     .setRange(0,255)
     .setValue(greenVal)
     .setCaptionLabel("Green")
     .setGroup(g4)
     ;
  
  cp5.getController("greenSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("greenSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
  
  cp5.addSlider("blueSlider")
     .setPosition(170, 40)
     .setSize(50,100)
     .setColor(blue1Colour)
     .setRange(0,255)
     .setValue(blueVal)
     .setCaptionLabel("Blue")
     .setGroup(g4)
     ;
  
  cp5.getController("blueSlider").getValueLabel().align(ControlP5.LEFT, ControlP5.LEFT_OUTSIDE).setPaddingX(-20);
  cp5.getController("blueSlider").getCaptionLabel().align(ControlP5.CENTER, ControlP5.BOTTOM_OUTSIDE);
     
  cp5.addButton("eye_button")
     .setValue(0)
     .setPosition(260,60)
     .setSize(90,50)
     .setGroup(g4)
     .setColor(mix1Colour)
     .setCaptionLabel("All on / off")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
  
  
  // -- LDRS -- //
  
  Group g5 = cp5.addGroup("g5")
                .setPosition(620,270)
                .setWidth(380)
                .setHeight(390)
                .setBackgroundHeight(390)
                .activateEvent(true)
                .setBackgroundColor(color(255,80))
                .setLabel("")
                .disableCollapse()
                .hideBar()
                ;
  
  cp5.addTextlabel("ldrs")
     .setText("Photocells")
     .setPosition(120,7)
     .setColorValue(titleCol[chosenTheme])
     .setGroup(g5)
     .setFont(titleFontc5)
     ;
  
  cp5.addTextlabel("leftldr")
     .setText("LEFT")
     .setPosition(100,300)
     .setGroup(g5)
     ;
  
  cp5.addTextlabel("rightldr")
     .setText("RIGHT")
     .setPosition(200,300)
     .setGroup(g5)
     ;
     
  cp5.addButton("enable_ldrs_button")
     .setValue(0)
     .setPosition(90,330)
     .setSize(200,30)
     .setGroup(g5)
     .setColor(buttonColour)
     .setCaptionLabel("Enable")
     .getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER)
     ;
  
}


// -- GUI -- //

public void controlEvent(ControlEvent theEvent) {
  // DropdownList is of type ControlGroup.
  // A controlEvent will be triggered from inside the ControlGroup class.
  // therefore you need to check the originator of the Event with
  // if (theEvent.isGroup())
  // to avoid an error message thrown by controlP5.

  if (theEvent.isGroup()) {
    // check if the Event was triggered from a ControlGroup
    //println("event from group : "+theEvent.getGroup().getValue()+" from "+theEvent.getGroup());
    
    if(theEvent.getGroup().getName().equals("serialports")) {
      port = (int)theEvent.getGroup().getValue();
      println("selected port: " + port);
    }
    
  } 
  else if (theEvent.isController()) {
    //println("event from controller : "+theEvent.getController().getValue()+" from "+theEvent.getController());
  }
}


// -- TITLE -- //

public void customize(DropdownList ddl) {
  // a convenience function to customize a DropdownList
  ddl.setBackgroundColor(color(190));
  ddl.setItemHeight(30);
  //ddl.setBarHeight(15);
  ddl.getCaptionLabel().align(ControlP5.CENTER, ControlP5.CENTER);
  //ddl.captionLabel().style().marginTop = 3;
  //ddl.captionLabel().style().marginLeft = 3;
  ddl.valueLabel().style().marginTop = 3;
}

public void connect(int theValue) {
  
  if(!connected) {
  
  if(port != 99) {
    println("connected");
    connected = true;
    arduino = new Serial(this, Serial.list()[port], 9600);
    cp5.getController("connect").setCaptionLabel("Disconnect");
  } else {
   println("need to select a port!"); 
  }
  
  } else {
    
    arduino.clear();
    arduino.stop();
    
    cp5.getController("connect").setCaptionLabel("Connect!");
    //port = 99;
    connected = false;
    
  }
  
}


// -- THEMES -- //

public void estop_button(int theValue) {
  
  estop = !estop;
  
  if(estop) {
    ((Button)cp5.getController("estop_button")).setCaptionLabel("Reset");
  } else {
    ((Button)cp5.getController("estop_button")).setCaptionLabel("Emergency Stop");
  }
  
}

public void theme1_button(int theValue) {
  chosenTheme = 0;
  updateTheme();
}

public void theme2_button(int theValue) {
  chosenTheme = 1;
  updateTheme();
}

public void theme3_button(int theValue) {
  chosenTheme = 2;
  updateTheme();
}


public void updateTheme() {
  
  if(DEBUG) println("updating theme!");
  
  buttonColour = new CColor(buttonFg[chosenTheme], buttonBg[chosenTheme], buttonAct[chosenTheme], captionCol[chosenTheme], valueCol[chosenTheme]);
  
  if(cp5.getController("title") != null) cp5.getController("title").setColorValue(titleCol[chosenTheme]);
  if(cp5.getController("servos") != null) cp5.getController("servos").setColorValue(titleCol[chosenTheme]);
  if(cp5.getController("positions") != null) cp5.getController("positions").setColorValue(titleCol[chosenTheme]);
  if(cp5.getController("actions") != null) cp5.getController("actions").setColorValue(titleCol[chosenTheme]);
  if(cp5.getController("leds") != null) cp5.getController("leds").setColorValue(titleCol[chosenTheme]);
  if(cp5.getController("ldrs") != null) cp5.getController("ldrs").setColorValue(titleCol[chosenTheme]);
  
  for(int i=0; i<allThemedElements.length; i++) {
    if(cp5.getController(allThemedElements[i]) != null) cp5.getController(allThemedElements[i]).setColor(buttonColour);
  }
  
  if(d1 != null) d1.setColor(buttonColour);
  
  saveXml();
  
}


// -- SLIDERS -- //

public void sendSerial(String s, int n) {
  
}

public void beakSlider(int s) {
  beakPos = s;
  if(DEBUG) println("beak: " + beakPos);
  transmit_action('@', 'B', 5, beakPos, '!');
}

public void leftSlider(int s) {
  leftPos = s;
  if(DEBUG) println("left wing: " + leftPos);
  transmit_action('@', 'L', 5, leftPos, '!');
}

public void rightSlider(int s) {
  rightPos = s;
  if(DEBUG) println("right wing: " + rightPos);
  transmit_action('@', 'R', 5, rightPos, '!');
}


// -- TEXTFIELDS -- //

public void beakopen(String theText) {
  beakOpenPos = (int)parseInt(theText);
  if(DEBUG) println("beakopen: "+ beakOpenPos);
  saveXml();
}

public void beakclosed(String theText) {
  beakClosedPos = (int)parseInt(theText);
  if(DEBUG) println("beakclosed: "+ beakClosedPos);
  saveXml();
}


public void leftup(String theText) {
  lwingUpPos = (int)parseInt(theText);
  if(DEBUG) println("leftup: "+ lwingUpPos);
  saveXml();
}

public void leftdown(String theText) {
  lwingDownPos = (int)parseInt(theText);
  if(DEBUG) println("leftdown: "+ lwingDownPos);
  saveXml();
}

public void lefthome(String theText) {
  lwingHomePos = (int)parseInt(theText);
  if(DEBUG) println("lefthome: "+ lwingHomePos);
  saveXml();
}


public void rightup(String theText) {
  rwingUpPos = (int)parseInt(theText);
  if(DEBUG) println("rightup: "+ rwingUpPos);
  saveXml();
}

public void rightdown(String theText) {
  rwingDownPos = (int)parseInt(theText);
  if(DEBUG) println("rightdown: "+ rwingDownPos);
  saveXml();
}

public void righthome(String theText) {
  rwingHomePos = (int)parseInt(theText);
  if(DEBUG) println("righthome: "+ rwingHomePos);
  saveXml();
}


public void writevals_button(int theValue) {
  println("writeevals pressed");
  saveXml();
  
  println("Writing the values to RoboBrrd's EEPROM...");
  
  // there will be a delay of ~200ms while all of the messages transfer over
  
  //transmit_action('^', 'E', 1, beak servo home, '!'); // beak servo home
  //delay(20);
  
  transmit_action('^', 'E', 2, rwingHomePos, '!'); // right wing servo home
  delay(20);
  
  transmit_action('^', 'E', 3, lwingHomePos, '!'); // left wing servo home
  delay(20);
  
  transmit_action('^', 'E', 5, beakOpenPos, '!'); // beak servo open
  delay(20);
  
  transmit_action('^', 'E', 6, rwingUpPos, '!'); // right wing servo up
  delay(20);
  
  transmit_action('^', 'E', 7, lwingUpPos, '!'); // left wing servo up
  delay(20);
  
  transmit_action('^', 'E', 9, beakClosedPos, '!'); // beak servo closed
  delay(20);
  
  transmit_action('^', 'E', 10, rwingDownPos, '!'); // right wing down
  delay(20);
  
  transmit_action('^', 'E', 11, lwingDownPos, '!'); // left wing down
  delay(20);
  
  transmit_action('^', 'E', 14, 0, '!'); // leds
  delay(20);
  
  println("...Done writing!");
  
}


// -- BUTTONS -- //

public void beakopen_button(int theValue) {
  if(DEBUG) println("beakopen pressed");
  transmit_action('#', 'B', 0, 0, '!');
}

public void beakclosed_button(int theValue) {
  if(DEBUG) println("beakclosed pressed");
  transmit_action('#', 'B', 1, 0, '!');
}


public void lwingup_button(int theValue) {
  if(DEBUG) println("lwingup pressed");
  transmit_action('#', 'L', 0, 0, '!');
}

public void lwingdown_button(int theValue) {
  if(DEBUG) println("lwingdown pressed");
  transmit_action('#', 'L', 1, 0, '!');
}

public void lwinghome_button(int theValue) {
  if(DEBUG) println("lwinghome pressed");
  transmit_action('#', 'L', 2, 0, '!');
}


public void rwingup_button(int theValue) {
  if(DEBUG) println("rwingup pressed");
  transmit_action('#', 'R', 0, 0, '!');
}

public void rwingdown_button(int theValue) {
  if(DEBUG) println("rwingdown pressed");
  transmit_action('#', 'R', 1, 0, '!');
}

public void rwinghome_button(int theValue) {
  if(DEBUG) println("rwinghome pressed");
  transmit_action('#', 'R', 2, 0, '!');
}


// -- LEDs -- //

public void redSlider(int s) {
  redVal = s;
  if(DEBUG) println("red: " + redVal);
  transmit_action('@', 'E', 0, redVal, '!');
  last_eye_change = millis();
  eye_changed = true;
  //saveXml();
}

public void greenSlider(int s) {
  greenVal = s;
  if(DEBUG) println("green: " + greenVal);
  transmit_action('@', 'E', 1, greenVal, '!');
  last_eye_change = millis();
  eye_changed = true;
  //saveXml();
}

public void blueSlider(int s) {
  blueVal = s;
  if(DEBUG) println("blue: " + blueVal);
  transmit_action('@', 'E', 2, blueVal, '!');
  last_eye_change = millis();
  eye_changed = true;
  //saveXml();
}

public void eye_button(int theValue) {
  if(DEBUG) println("eye button pressed");
  
  if(eye_press) {
    redSlider(255);
    delay(20);
    greenSlider(255);
    delay(20);
    blueSlider(255);
    delay(20);
    
    mix1Colour.setBackground(white);
    mix1Colour.setForeground(black);
    
  } else {
    redSlider(0);
    delay(20);
    greenSlider(0);
    delay(20);
    blueSlider(0);
    delay(20);
    
    mix1Colour.setBackground(black);
    mix1Colour.setForeground(white);
    
  }
  
  eye_press = !eye_press;
  
  cp5.getController("eye_button").setColor(mix1Colour);
  
  
  ((Slider)cp5.getController("redSlider")).setValue(redVal);
  ((Slider)cp5.getController("greenSlider")).setValue(greenVal);
  ((Slider)cp5.getController("blueSlider")).setValue(blueVal);
  
  last_eye_change = millis();
  eye_changed = true;
  
}


// -- LDRS -- //

public void enable_ldrs_button(int theValue) {
  if(DEBUG) println("enable_ldrs_button pressed");
  refresh_ldrs = !refresh_ldrs;
  
  if(refresh_ldrs) {
    ((Button)cp5.getController("enable_ldrs_button")).setCaptionLabel("Disable");  
  } else {
    ((Button)cp5.getController("enable_ldrs_button")).setCaptionLabel("Enable"); 
  }
  
}


  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "RoboBrrd_Dashboard" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
